#include "GEOMImpl_Gen.hxx"
#include "SMESHDS_Mesh.hxx"
#include "SMESH_ControlsDef.hxx"
#include "SMESH_Gen.hxx"
#include "SMESH_Mesh.hxx"
#include "StdMeshers_Hexa_3D.hxx"
#include "StdMeshers_NumberOfSegments.hxx"
#include "StdMeshers_Quadrangle_2D.hxx"
#include "StdMeshers_Regular_1D.hxx"

#include <emscripten/bind.h>

// Define easier interfaces to use in Javascript

double getTotalVolume(SMESH::Controls::Volume &volControl, SMESH_Mesh *mesh) {
  double vol = 0;
  SMDS_Mesh *ds = static_cast<SMDS_Mesh *>(mesh->GetMeshDS());
  volControl.SetMesh(ds);
  for (int elemID = 0; elemID <= ds->NbElements(); elemID++) {
    vol += volControl.GetValue(elemID);
  }
  return vol;
}

const std::string addHypothesis(SMESH_Mesh &mesh, TopoDS_Shape &shape, int id) {
  std::string e;
  mesh.AddHypothesis(shape, id, &e);
  return e;
}

bool compute(SMESH_Gen &gen, SMESH_Mesh *mesh, TopoDS_Shape &shape) {
  return gen.Compute(*mesh, shape);
}

EMSCRIPTEN_BINDINGS(smesh) {
  // OCCT bindings
  emscripten::class_<TopoDS_Shape>("CAS_Shape").constructor<>();

  // GEOM Bindings
  emscripten::class_<GEOMImpl_Gen>("GEOM_Gen")
      .constructor<>()
      .function("GetI3DPrimOperations", &GEOMImpl_Gen::GetI3DPrimOperations,
                emscripten::allow_raw_pointers());

  // CAS Pointer
  emscripten::class_<Handle_GEOM_Object>("Handle_GEOM_Object")
      .function("get", &Handle_GEOM_Object::get,
                emscripten::allow_raw_pointers());

  emscripten::class_<GEOM_Object>("GEOM_Object")
      .function("GetValue", &GEOM_Object::GetValue);

  emscripten::class_<GEOMImpl_I3DPrimOperations>("GEOM_PrimOperations")
      .function("MakeBoxDXDYDZ", &GEOMImpl_I3DPrimOperations::MakeBoxDXDYDZ);

  // SMESH Bindings
  emscripten::class_<SMESH_Gen>("SMESH_Gen")
      .constructor<>()
      .function("CreateMesh", &SMESH_Gen::CreateMesh,
                emscripten::allow_raw_pointers())
      .function("Compute", &compute, emscripten::allow_raw_pointers());

  emscripten::class_<SMESHDS_Mesh>("SMESH_DS_Mesh")
      .function("NbElements", &SMESHDS_Mesh::NbElements);
  emscripten::class_<SMDS_Mesh>("SMESH_SMDS_Mesh");

  emscripten::class_<SMESH_Hypothesis>("SMESH_Hypothesis")
      .class_function("IsStatusFatal", &SMESH_Hypothesis::IsStatusFatal);

  emscripten::class_<SMESH_Mesh>("SMESH_Mesh")
      .function("ShapeToMesh", &SMESH_Mesh::ShapeToMesh)
      .function("AddHypothesis", &addHypothesis,
                emscripten::allow_raw_pointers());
  // Meshers
  emscripten::class_<StdMeshers_Regular_1D>("SMESH_Meshers_Regular_1D")
      .constructor<int, SMESH_Gen *>();
  emscripten::class_<StdMeshers_NumberOfSegments>(
      "SMESH_Meshers_NumberOfSegments")
      .constructor<int, SMESH_Gen *>()
      .function("SetNumberOfSegments",
                &StdMeshers_NumberOfSegments::SetNumberOfSegments);
  emscripten::class_<StdMeshers_Quadrangle_2D>("SMESH_Meshers_Quadrangle_2D")
      .constructor<int, SMESH_Gen *>();
  emscripten::class_<StdMeshers_Hexa_3D>("SMESH_Meshers_Hexa_3D")
      .constructor<int, SMESH_Gen *>();

  // Measurements
  emscripten::class_<SMESH::Controls::Volume>("SMESH_Controls_Volume")
      .constructor<>()
      .function("GetTotal", &getTotalVolume, emscripten::allow_raw_pointers());
}
